/*
**
** Sarastusvalon ohjain (c) JTA 2013-11-23
**
** Compiled with HI-TECH C Compiler for PIC10-12-16 MCUs V9.83
**
*/

#include <htc.h>
#include <stdint.h>
#include "hardware.h"

__CONFIG(DEBUG_OFF & CP_OFF & WDTE_OFF & BOREN_ON & PWRTE_ON & FOSC_XT & LVP_OFF & CPD_OFF & WRT_OFF);

#define RAMPTIME_READ_COUNT (10)

void hwinit(void);

volatile uint16_t flags = 0;

void main(void)
{
    uint16_t u16PWM = 0;
    uint32_t u32Brightness = 0, u32BrightnessStep = 0;
    uint8_t u8Stop = 0;
    uint8_t u8RampTime = 0;
    uint8_t u8ReadCount = 0;
    uint8_t u8CurrPortB = 0, u8LastPortB = 0;
 
    flags = 0;
    hwinit();

    /*
    ** Read ramp time setting at least RAMPTIME_READ_COUNT times, accept if all reads are the same
    */
    
    u8LastPortB = ~PORTB; 
    
    while(1)
    {
        __delay_ms(100);
        u8CurrPortB = ~PORTB;
        if (u8CurrPortB == u8LastPortB)
        {
            u8ReadCount++;
            if (RAMPTIME_READ_COUNT == u8ReadCount)
            {
                u8RampTime = u8CurrPortB;
                break;
            }
        }
        else
        {
            u8ReadCount = 0;
        }
        u8LastPortB = u8CurrPortB;
    }

    /*
    ** Calculate PWM increment per loop, set straight to maximum if ramp time set to 0
    */

    if (u8RampTime > 0)
    {
        u32BrightnessStep = (1048576UL*1024UL/(u8RampTime*234375UL));
    }
    else
    {
        u32BrightnessStep = (1024UL<<16);
    }

    /*
    ** Start PWM and timer 2 interrupts
    */

    T2CONbits.TMR2ON = 1;
    PEIE = 1;
    GIE = 1;

    /*
    ** Do the business, set u8Stop flag when maximum PWM setting reached
    */

    while(1)
    {
        if (flags & FLG_TICK)
        {
            if (!u8Stop)
            {
                u32Brightness += u32BrightnessStep;
                u16PWM = (u32Brightness >> 16);
                if (u16PWM > 1023)
                {
                    u8Stop = 1;
                    u16PWM = 0x3FF;
                }
                CCPR1L = (u16PWM >> 2) & 0xFF;
                CCP1CONbits.CCP1X = (u16PWM >> 1) & 0x1;
                CCP1CONbits.CCP1Y = u16PWM & 0x1;
            }
            flags &= ~FLG_TICK;
        }
    }
}

void hwinit(void)
{
    PORTA = 0;
    PORTB = 0;
    PORTC = 0;
    PORTD = 0;
    PORTE = 0;

    /* ADC is not used, all pins as digital */
    ADCON0 = 0x00;
    ADCON1 = 0x06;

    nRBPU = 0;
    TRISA = 0x00;
    TRISB = 0xFF;
    TRISC = 0x00;
    TRISD = 0x00;
    TRISE = 0x00;
    CCP1CONbits.CCP1X = 0;
    CCP1CONbits.CCP1Y = 0;
    CCP1CONbits.CCP1M = 0xC;
    CCPR1L = 0x00;
    PR2 = 0xFF;
    T2CONbits.T2CKPS = 0;
    T2CONbits.TOUTPS = 0xF; /* postscale = 16, TMR2 interrupt frequency 1220 Hz @ 20 MHz */
    PIR1bits.TMR2IF = 0;
    PIE1bits.TMR2IE = 1;
}

void interrupt tick_int()
{
    if (TMR2IE && TMR2IF)
    {
        flags |= FLG_TICK;
        TMR2IF = 0;
    }
}